# -*- coding: utf-8 -*-
"""
/***************************************************************************
 BGTInloopTool
                                 A QGIS plugin
 BGT Inlooptool
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2020-08-12
        git sha              : $Format:%H$
        copyright            : (C) 2020 by Emile de Badts
        email                : emile.debadts@nelen-schuurmans.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import ogr

from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction
from qgis.core import Qgis, QgsProject
from qgis.utils import iface


from qgis.core import (
  QgsProcessingContext,
  QgsTaskManager,
  QgsTask,
  QgsProcessingAlgRunnerTask,
  Qgis,
  QgsProcessingFeedback,
  QgsApplication,
  QgsMessageLog,
)

# Initialize Qt resources from file resources.py
from .resources import *
# Import the code for the dialog
from .BGTInloopTool_dialog import BGTInloopToolDialog
import os.path, sys

# Import the BGT Inlooptool core
sys.path.append(os.path.dirname(os.path.realpath(__file__)))
from core.inlooptool import *
from .ogr2qgis import *

MESSAGE_CATEGORY = 'BGT Inlooptool'
INLOOPTABEL_STYLE = os.path.join(os.path.dirname(__file__), 'bgt_inlooptabel.qml')

class InloopToolTask(QgsTask):
    
    def __init__(self, description, parameters, bgt_file, pipe_file, building_file, use_index):
        super().__init__(description, QgsTask.CanCancel)
        
        QgsMessageLog.logMessage("initalized task", MESSAGE_CATEGORY, level=Qgis.Info)

        self.parameters = parameters
        self.bgt_file = bgt_file
        self.pipe_file = pipe_file
        self.building_file = building_file
        self.use_index = use_index
        self.exception = None
        
    def run(self):

        QgsMessageLog.logMessage("started inlooptool task", MESSAGE_CATEGORY, level=Qgis.Info)        
        self.it = InloopTool(self.parameters)
        
        #database_fn = 'C:/Users/Emile.deBadts/Documents/Projecten/v0099_bgt_inlooptool/mem_database.gpkg'
        #self.it._database.mem_database = ogr.Open(database_fn,1)
        
        # Import surfaces and pipes and calculate ruonff targets
        QgsMessageLog.logMessage("importing pipes", MESSAGE_CATEGORY, level=Qgis.Info)        
        self.it.import_surfaces(self.bgt_file)
        QgsMessageLog.logMessage("importing surfaces", MESSAGE_CATEGORY, level=Qgis.Info)        
        self.it.import_pipes(self.pipe_file)    
        QgsMessageLog.logMessage("importing buildings", MESSAGE_CATEGORY, level=Qgis.Info)        
        self.it.import_buildings(self.building_file)    
        self.it._database.add_build_year_to_surface(use_index = self.use_index)
        QgsMessageLog.logMessage("calculating distances", MESSAGE_CATEGORY, level=Qgis.Info)        
        self.it.calculate_distances(parameters = self.parameters, use_index = self.use_index)
        QgsMessageLog.logMessage("calculating runoff targets", MESSAGE_CATEGORY, level=Qgis.Info)        
        self.it.calculate_runoff_targets()
        
        # Export database
        #self.it._database._write_to_disk(database_fn)
        
        return(True)
            
    def finished(self, result):
        
        if result:
            QgsMessageLog.logMessage("successfully calculated runoff targets", MESSAGE_CATEGORY, level=Qgis.Info)
    
            # Exporteren output naar QGIS layer
            ogr_lyr = self.it._database.mem_database.GetLayerByName('bgt_inlooptabel')
            if ogr_lyr is not None:
                if ogr_lyr.GetFeatureCount() > 0:
                    qgs_lyr = as_qgis_memory_layer(ogr_lyr, 'BGT Inlooptabel')
                    project = QgsProject.instance()
                    project.addMapLayer(qgs_lyr)
                    layer =QgsProject.instance().mapLayersByName('BGT Inlooptabel')[0]
                    layer.loadNamedStyle(INLOOPTABEL_STYLE)
                    layer.triggerRepaint()
        
        else:
            if self.exception is None:
                QgsMessageLog.logMessage("task failed: cancelled by user", MESSAGE_CATEGORY, level=Qgis.Critical)
            else:
                QgsMessageLog.logMessage("task failed: {}".format(self.exception), MESSAGE_CATEGORY, level=Qgis.Critical)
                raise self.exception
    
    def cancel(self):
    
        QgsMessageLog.logMessage("task failed: cancelled by user", MESSAGE_CATEGORY, level=Qgis.Critical)
        super().cancel()
    


class BGTInloopTool:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        self.tm = QgsApplication.taskManager()

        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'BGTInloopTool_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&BGT Inlooptool')

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads
        self.first_start = None

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('BGTInloopTool', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/BGTInloopTool/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'Afwateringskenmerken BGT vlakken'),
            callback=self.run,
            parent=self.iface.mainWindow())

        # will be set False in run()
        self.first_start = True


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&BGT Inlooptool'),
                action)
            self.iface.removeToolBarIcon(action)

            
   
    def on_run(self):
        
        # input files            
        bgt_file = self.dlg.bgt_file.filePath()
        pipe_file =self.dlg.pipe_file.filePath() 
        building_file = self.dlg.building_file.filePath()
                    
        # Iniate bgt inlooptool class with parameters 
        parameters = InputParameters(max_afstand_vlak_afwateringsvoorziening = self.dlg.max_afstand_vlak_afwateringsvoorziening.value(), 
                                     max_afstand_vlak_oppwater = self.dlg.max_afstand_vlak_oppwater.value(), 
                                     max_afstand_pand_oppwater = self.dlg.max_afstand_pand_oppwater.value(), 
                                     max_afstand_vlak_kolk = self.dlg.max_afstand_vlak_kolk.value(), 
                                     max_afstand_afgekoppeld = self.dlg.max_afstand_afgekoppeld.value(),
                                     max_afstand_drievoudig = self.dlg.max_afstand_drievoudig.value(), 
                                     afkoppelen_hellende_daken = self.dlg.afkoppelen_hellende_daken.isChecked(), 
                                     bouwjaar_gescheiden_binnenhuisriolering = self.dlg.bouwjaar_gescheiden_binnenhuisriolering.value(), 
                                     verhardingsgraad_erf = self.dlg.verhardingsgraad_erf.value(), 
                                     verhardingsgraad_half_verhard = self.dlg.verhardingsgraad_half_verhard.value())
        
        inlooptooltask = InloopToolTask(description = 'Inlooptool task', 
                                        parameters = parameters,
                                        bgt_file = bgt_file,
                                        pipe_file = pipe_file,
                                        building_file  = building_file,
                                        use_index = USE_INDEX)
     
        self.tm.addTask(inlooptooltask)

                            

    def run(self):
        """Run method that performs all the real work"""

        # Create the dialog with elements (after translation) and keep reference
        # Only create GUI ONCE in callback, so that it will only load when the plugin is started
        if self.first_start == True:
            self.first_start = False
            
            if not USE_INDEX:
                iface.messageBar().pushMessage("Error", "rtree not installed, performance will be slower than optimal", level=Qgis.Error)

            self.dlg = BGTInloopToolDialog()
            # Initiating the tool in 'on_run'
            self.dlg.pushButtonRun.clicked.connect(self.on_run)
            
        # show the dialog
        self.dlg.show()
